Package: scTHI
Title: Indentification of significantly activated ligand-receptor
        interactions across clusters of cells from single-cell RNA
        sequencing data
Version: 1.24.0
Authors@R: c(person("Francesca Pia", "Caruso", email = "francescapia.caruso@gmail.com", role = c("aut")),
	person("Michele", "Ceccarelli", email = "m.ceccarelli@gmail.com", role = c("aut", "cre")))
Description: scTHI is an R package to identify active pairs of ligand-receptors from single cells in order to study,among others, tumor-host interactions. scTHI contains a set of signatures to classify cells from the tumor microenvironment.
Depends: R (>= 4.0)
License: GPL-2
Encoding: UTF-8
RoxygenNote: 6.1.1.9000
Imports: BiocParallel, Rtsne, grDevices, graphics, stats
Suggests: scTHI.data, knitr, rmarkdown, BiocStyle
VignetteBuilder: knitr
biocViews: Software,SingleCell
BugReports: https://github.com/miccec/scTHI/issues
git_url: https://git.bioconductor.org/packages/scTHI
git_branch: RELEASE_3_23
git_last_commit: c376b19
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:36:15 UTC; biocbuild
Author: Francesca Pia Caruso [aut],
  Michele Ceccarelli [aut, cre]
Maintainer: Michele Ceccarelli <m.ceccarelli@gmail.com>

# Generated by roxygen2: do not edit by hand

export(TME_classification)
export(TME_plot)
export(scTHI_plotCluster)
export(scTHI_plotPairs)
export(scTHI_plotResult)
export(scTHI_runTsne)
export(scTHI_score)
importFrom(BiocParallel,SnowParam)
importFrom(BiocParallel,bplapply)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,hcl)
importFrom(graphics,barplot)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,text)
importFrom(stats,density)
importFrom(stats,p.adjust)
importFrom(stats,pnorm)
importFrom(stats,quantile)
importFrom(stats,sd)

colByValue_ <-
  function(x,
           col,
           range = NA,
           breaks = NA,
           cex.axis = 2,
           las = 1,
           ...) {
    if (is.vector(x)) {
      x <- as.matrix(x)
    }
    if (is.na(range[1])) {
      
    }
    else {
      x[x < range[1]] <- range[1]
      x[x > range[2]] <- range[2]
    }
    if (is.na(breaks[1])) {
      ff <- seq(min(x, na.rm = TRUE), max(x, na.rm = TRUE),
                length = length(col)
      )
      bg2 <- apply(
        as.matrix(as.numeric(unlist(x))), 1,
        function(x) {
          rank(c(ff, x), ties.method = "min")[length(col) + 1]
        }
      )
      dens <- matrix(bg2, nrow(x), ncol(x))
      result <- matrix(col[dens], nrow = nrow(x), ncol = ncol(x))
      row.names(result) <- row.names(x)
      # image(x = 1:2, y = as.matrix(ff), z = t(ff), col = col,
      #       xaxt = "n", ylab = "", las = las, xlab = "",
      #       xlim = c(1, 4), bty = "n", ...)
      return(result)
    }
    else {
      temp <- cut(as.numeric(unlist(x)),
                  breaks = breaks,
                  include.lowest = TRUE
      )
      if (length(col) != length(levels(temp))) {
        stop("length:col != length: cut result")
      }
      result <- matrix(col[as.numeric(temp)],
                       nrow = nrow(x),
                       ncol = ncol(x)
      )
      row.names(result) <- row.names(x)
      return(result)
    }
  }



getcolors <- function(genesToplot, expMat, 
                      tsneData){
  
  ans <- lapply(genesToplot, function(x){
    tmp_exp <-
      t(expMat[x, colnames(expMat), drop = FALSE])
    mean_exp_samples <- mean(tmp_exp)
    
    ##### range gene x gene
    range_exp <- range(tmp_exp)
    bre <- seq(
      min(tmp_exp[, x]) - 0.01,
      max(tmp_exp[, x]) + 0.01, 0.01
    )
    colori <-
      colByValue_(
        tmp_exp,
        col = colorRampPalette(c(
          "gray80", "coral",
          "red"
        ))(length(bre) - 1),
        breaks = bre,
        cex.axis = 0.8
      )
    colori <- colori[rownames(tsneData), , drop = FALSE]
    tmp <- list(colori, range_exp, mean_exp_samples)
    names(tmp) <- c("colori", "range_exp", "mean_exp_samples")
    tmp})
  
  return(ans)
}


#' scTHI_plotPairs
#'
#' Generates a plot on the t-SNE coordinates to
#' show the expression levels
#' of an interaction pair of interest. Each cell is
#' colored according to the
#' corresponding
#' gene expression value.
#' @param scTHIresult scTHI object.
#' @param cexPoint Set the point size.
#' @param interactionToplot Interaction pair to plot.
#' @examples
#' library(scTHI.data)
#' data(scExample)
#' result <-  scTHI_score(scExample,
#'                  cellCusterA = colnames(scExample)[1:30],
#'                  cellCusterB = colnames(scExample)[31:100],
#'                  cellCusterAName = "ClusterA",
#'                  cellCusterBName = "ClusterB", filterCutoff = 0,
#'                  pvalueCutoff = 1, nPermu = 100, ncore = 8)
#' result <- scTHI_runTsne(result)
#' scTHI_plotPairs(result,interactionToplot = "CXCL12_CD4")

#' @return None
#' @export
#'
#' scTHI_plotPairs
scTHI_plotPairs <-
  function(scTHIresult,
           cexPoint = 0.8,
           interactionToplot) {
   
    tsneData <- scTHIresult$tsneData
    result <- scTHIresult$result
    expMat <- scTHIresult$expMat
    
    genesToplotA <- result[interactionToplot, c("partnerA")]
    genesToplotA <- unlist(strsplit(genesToplotA, ":"))
    genesToplotB <- result[interactionToplot, c("partnerB")]
    genesToplotB <- unlist(strsplit(genesToplotB, ":"))
    
    #################  create list colori
    list_colori_A <- getcolors(genesToplotA, expMat, tsneData)
    names(list_colori_A) <- genesToplotA
    
    list_colori_B <- getcolors(genesToplotB, expMat, tsneData)
    names(list_colori_B) <- genesToplotB
    
    ################# plot
    par(
      mfrow = c(1, length(c(
        genesToplotA, genesToplotB
      ))),
      mar = c(6, 4, 3, 2),
      cex.main = 1.5,
      cex.sub = 1.2,
      col.main = "black",
      col.sub = "gray30",
      font.main = 2,
      font.sub = 3
    )
    
    lapply(names(list_colori_A), function(x) {
      colori <- list_colori_A[[x]]$colori
      range_exp <- round(list_colori_A[[x]]$range_exp, digits = 2)
      mean_exp_samples <-
        round(list_colori_A[[x]]$mean_exp_samples, digits = 2)
      plot(
        tsneData[, 1:2],
        pch = 16,
        cex = cexPoint,
        col = colori,
        main = x,
        xlab = "",
        ylab = "",
        sub = paste0(
          "Range: ",
          range_exp[1],
          " to ",
          range_exp[2],
          ", MeanExp: ",
          mean_exp_samples
        )
      )
    })
    
    lapply(names(list_colori_B), function(x) {
      colori <- list_colori_B[[x]]$colori
      range_exp <- round(list_colori_B[[x]]$range_exp, digits = 2)
      mean_exp_samples <-
        round(list_colori_B[[x]]$mean_exp_samples, digits = 2)
      plot(
        tsneData[, 1:2],
        pch = 16,
        cex = cexPoint,
        col = colori,
        main = x,
        xlab = "",
        ylab = "",
        sub = paste0(
          "Range: ",
          range_exp[1],
          " to ",
          range_exp[2],
          ", MeanExp: ",
          mean_exp_samples
        )
      )
    })
  
    par(mfrow = c(1, 1), mar = c(5, 4, 4, 2) + 0.1)
  }

#' scTHI_plotResult
#'
#' Creates barplots of scTHI_score results.
#' @param scTHIresult scTHI object.
#' @param cexNames Size of names in barplot.
#' @param plotType Type of plot to be generated. Default is
#' "score", can be  also "pair". The "score" option will generate
#' a barplot for each resulted interaction pair, representing
#' the calculated interaction score
#'   and the related p-Value.The "pair" option will generate
#'   two barplot for each resulted interaction pair, representing
#'   the percentage of cells of each cluster expressing partnerA
#'   and partnerB gene, respectively.
#' @param nRes Number of pairs to plot (all if NULL).
#' @examples
#' library(scTHI.data)
#' data(scExample)
#' result <-  scTHI_score(scExample,
#'                        cellCusterA = colnames(scExample)[1:30],
#'                        cellCusterB = colnames(scExample)[31:100],
#'                        cellCusterAName = "ClusterA",
#'                        cellCusterBName = "ClusterB", filterCutoff = 0,
#'                        pvalueCutoff = 1, nPermu = 100, ncore = 8)
#'
#' scTHI_plotResult(result, plotType = "score")
#' scTHI_plotResult(result, plotType = "pair")
#' @return None
#' @export
#'
#' scTHI_plotResult

scTHI_plotResult <- function(scTHIresult,
                             cexNames = 0.8,
                             plotType = c("score", "pair"),
                             nRes = NULL) {
  
  result <- scTHIresult$result
  if (!is.null(nRes)) {
    result <- result[1:nRes, ]
  }
  
  
  if (plotType == "score") {
    tmp <- as.matrix(result[, c("SCORE", "pValue")])
    pvalues <- apply(tmp, 1, function(x) {
      if (x["pValue"] > 0.05) "ns" else if (x["pValue"] <= 0.0001){ 
        "****"
      }else if (x["pValue"] <= 0.001) {
        "***"
      }else if (x["pValue"] <= 0.01) {
        "**"
      }else if (x["pValue"] <= 0.05) {
        "*"
      }
      
    })
    
    par(mar = c(5, 10, 4, 2) + 0.1)
    barplot(
      t(tmp[, "SCORE"]),
      beside = TRUE,
      xlim = c(0, 1.1),
      xlab = "scTHI Score",
      col = c("lightseagreen"),
      cex.names = cexNames,
      horiz = TRUE,
      las = 2
    )
    text(
      x = 1.05,
      y = seq(1.5, by = 2, length.out = nrow(tmp)),
      labels = pvalues,
      cex = 1.2
    )
    par(mar = c(5, 4, 4, 2) + 0.1)
  }
  
  if (plotType == "pair") {
    tmp <- as.matrix(result[, grep("rnk", colnames(result))])
    par(mar = c(5, 10, 4, 2) + 0.1, oma = c(0, 0, 0, 6))
    barplot(
      t(tmp),
      beside = TRUE,
      xlim = c(0, 1.1),
      xlab = "% of Cells",
      col = c("#F8766D", "#00BFC4"),
      cex.names = cexNames,
      horiz = TRUE,
      las = 2
    )
    legend(
      par("usr")[2],
      par("usr")[4],
      bty = "n",
      xpd = NA,
      legend = c("ClusterA", "ClusterB"),
      col = c("#F8766D", "#00BFC4"),
      cex = 1,
      pch = 15,
      box.lty = 0,
      bg = "transparent"
    )
    par(mar = c(5, 4, 4, 2) + 0.1, oma = c(0, 0, 0, 0))
  }
}

#' TME_plot
#'
#' Generates a plot on the t-SNE coordinates, labeling cells by TME
#' classification.
#'
#' @param tsneData X and y coordinates of points in the plot.
#' @param Class Object returned by TME_classification function.
#' @param cexPoint Set the point size.
#' @examples
#' library(scTHI.data)
#' data(scExample)
#' result <-  scTHI_score(scExample,
#'            cellCusterA = colnames(scExample)[1:30],
#'            cellCusterB = colnames(scExample)[31:100],
#'            cellCusterAName = "ClusterA",
#'            cellCusterBName = "ClusterB", filterCutoff = 0,
#'            pvalueCutoff = 1, nPermu = 100, ncore = 8)
#' result <- scTHI_runTsne(result)
#' Class <- TME_classification(scExample)
#' TME_plot(tsneData = result$tsneData, Class)
#' @return  None
#' @export
#'
#' TME_plot

TME_plot <- function(tsneData, Class, cexPoint = 0.8) {
  
  ClassColor <- Class$ClassLegend[Class$Class]
  names(ClassColor) <- names(Class$Class)
  
  TmeColors <- rep("gray80", nrow(tsneData))
  names(TmeColors) <- rownames(tsneData)
  TmeColors[names(ClassColor)] <- ClassColor
  par(oma = c(0, 0, 0, 6))
  plot(
    tsneData[, 1:2],
    pch = 16,
    cex = cexPoint,
    col = TmeColors[rownames(tsneData)],
    main = "",
    xlab = "tSNE 1",
    ylab = "tSNE 2"
  )
  legend(
    par("usr")[2],
    par("usr")[4],
    bty = "n",
    xpd = NA,
    legend = names(Class$ClassLegend),
    col = Class$ClassLegend,
    cex = 0.9,
    pch = 15,
    box.lty = 0
  )
}

#' @importFrom grDevices colorRampPalette hcl
#' @importFrom graphics barplot legend par plot text
#' @importFrom stats density p.adjust quantile sd pnorm
#' @importFrom BiocParallel SnowParam bplapply
NULL

#' scTHI_runTsne
#'
#' Runs t-SNE dimensionality reduction on selected features based on Rtsne
#' package.
#' @param scTHIresult scTHI object.
#' @examples
#' library(scTHI.data)
#' data(scExample)
#' result <-  scTHI_score(scExample,
#'                        cellCusterA = colnames(scExample)[1:30],
#'                        cellCusterB = colnames(scExample)[31:100],
#'                        cellCusterAName = "ClusterA",
#'                        cellCusterBName = "ClusterB", filterCutoff = 0,
#'                        pvalueCutoff = 1, nPermu = 100, ncore = 8)
#' result <- scTHI_runTsne(result)
#' @return The same object as scTHI_score with a fifth item tsneData
#'   (data.frame)
#' @export
#'
#' scTHI_runTsne

scTHI_runTsne <- function(scTHIresult) {

  expMat <- scTHIresult$expMat
  
  ## create tsne
  eta <- .1
  filter <- apply(expMat, 1, function(x) {
    sum(quantile(x,
                 probs = c(1 - eta, eta)
    ) * c(1, -1))
  })
  # fivenum(filter)
  # plot(density(log2(filter)))
  cutoff <- density(log2(filter))
  cutoff <- data.frame(x = cutoff$x, y = cutoff$y)
  cutoff <- cutoff$x[which.max(cutoff$y)]
  foldChange <- 2^cutoff
  # sum(filter > foldChange) # 4567
  variableGenes <- names(filter)[filter > foldChange]
  expMat <- expMat[variableGenes, ]
  
  requireNamespace("Rtsne")
  expMatT <- t(expMat)
  tsne_out <- Rtsne::Rtsne(expMatT)
  tsneData <- data.frame(
    x = tsne_out$Y[, 1],
    y = tsne_out$Y[, 2],
    Sample = colnames(expMat),
    stringsAsFactors = FALSE
  )
  rownames(tsneData) <- colnames(expMat)
  scTHIresult <- c(list(tsneData), scTHIresult)
  names(scTHIresult)[1] <- "tsneData"
  return(scTHIresult)
}

#' TME_classification
#'
#' The function allows the user to classify non-tumor cells in tumor
#' microenvironment. It implements the Mann-Whitney-Wilcoxon Gene
#' Set Test
#' (MWW-GST) algorithm and tests for each cell the enrichment of
#' a collection
#' of signatures of different cell types.
#' @param expMat Gene expression matrix where rows are genes
#'   presented with
#'   Hugo Symbols and columns are cells. Gene expression values
#'   should be normalized counts.
#' @param minLenGeneSet Minimum gene set length
#' @param pvalFilter Logical, if TRUE results will be filtered
#' for p-Value.
#'   Defoult is FALSE.
#' @param alternative a character string specifying the alternative
#' hypothesis of wilcoxon test, must be one of "two.sided" (default),
#' "greater" or "less".
#' @param fdrFilter Logical, if TRUE results will be filtered for FDR.
#' @param pvalCutoff Numeric p-Value (or FDR) threshold. Gene set with
#'   p-Value (or FDR) greater than pvalCutoff will be discarded
#'   (default is 0.01).
#' @param nesCutoff Numeric threshold. Gene set with NES greater than
#'   nesCutoff will be discarded (default is 0.58)
#' @param nNES Default is 0.58, so each cell is classified with
#'  a specific
#'   phenotype based on the first significant enriched gene set.
#' @examples
#' library(scTHI.data)
#' data(scExample)
#' Class <- TME_classification(scExample)
#' @return A list with two items: Class (character) and ClassLegend
#'   (character)
#' @export
#'
#' TME_classification

TME_classification <- function(expMat,
                               minLenGeneSet = 10,
                               alternative = "two.sided",
                               pvalFilter = FALSE,
                               fdrFilter = TRUE,
                               pvalCutoff = 0.01,
                               nesCutoff = 0.58,
                               nNES = 1) {
  
  zerogenes <- rowSums(expMat == 0)
  expMat <- expMat[zerogenes != ncol(expMat), ]
  means <- rowMeans(expMat)
  sds <- apply(expMat, 1, sd)
  E <- apply(expMat, 2, function(x) {
    (x - means) / sds
  })
  
  E_ <- apply(E, 2, rank)
  tmp <- lapply(signatures, function(x) {
    sum(x %in% rownames(E))
  })
  signatures_ <- signatures[tmp > minLenGeneSet]
  
  ans <- vector("list", length(signatures_))
  names(ans) <- names(signatures_)
  ans <- lapply(signatures_, function(S) {
    geneSet <- S
    gs <- geneSet[which(geneSet %in% rownames(E))]
    outside_gs <- setdiff(rownames(E), gs)
    nx <- length(gs)
    ny <- length(outside_gs)
    Tstat <- colSums(E_[outside_gs, ])
    Ustat <- nx * ny + ny * (ny + 1) / 2 - Tstat
    mu <- nx * ny / 2
    sigma <- sqrt(mu * (nx + ny + 1) / 6)
    zValue <- Ustat - mu
    correction <- vapply(zValue, function(x) {
      switch(
        alternative,
        two.sided = sign(x) * 0.5,
        greater = 0.5,
        less = -0.5
      )
    },numeric(1))
    zValue <- (zValue - correction) / sigma
    pValue <- vapply(zValue, function(x) {
      switch(
        alternative,
        less = 1 - pnorm(x),
        greater = pnorm(-x),
        two.sided = 2 * pnorm(-abs(x))
      )
    },numeric(1))
    nes <- Ustat / nx / ny
    pu <- nes / (1 - nes)
    log.pu <- log2(pu)
    names(Ustat) <- colnames(E_)
    names(pValue) <- colnames(E_)
    names(nes) <- colnames(E_)
    names(pu) <- colnames(E_)
    names(log.pu) <- colnames(E_)
    #ans[[i]] <- 
    list(
      statistic = Ustat,
      p.value = pValue,
      nes = nes,
      pu = pu,
      log.pu = log.pu
    )})
  
  
  NES <- vapply(ans, function(x) {
    cbind(x$log.pu)
  },numeric(ncol(expMat)))
  NES <- t(NES)
  colnames(NES) <- colnames(expMat)
  pValue <- vapply(ans, function(x) {
    cbind(x$p.value)
  },numeric(ncol(expMat)))
  pValue <- t(pValue)
  colnames(pValue) <- colnames(expMat)
  
  FDR <- apply(pValue, 2, function(x) {
    p.adjust(x, method = "fdr")
  })
  if (pvalFilter == TRUE) {
    NES[pValue >= pvalCutoff] <- 0
  }
  if (fdrFilter == TRUE) {
    NES[FDR >= pvalCutoff] <- 0
  }
  NES[NES < nesCutoff] <- 0
  if (nNES == 1) {
    Class <- apply(NES, 2, function(x) {
      names(which.max(x))
    })
  }
  if (nNES != 1) {
    Class <- apply(NES, 2, function(x) {
      names(sort(x, decreasing = TRUE))[nNES]
    })
  }
  Class[colSums(NES != 0) < nNES] <- "nc"
  phenotype <- signaturesColors[Class, ]
  rownames(phenotype) <- names(Class)
  Class <- phenotype$ALLPhenotypeFinal
  names(Class) <- rownames(phenotype)
  ClassLegend <- phenotype$Color
  names(ClassLegend) <- phenotype$ALLPhenotypeFinal
  ClassLegend <- ClassLegend[!duplicated(ClassLegend)]
  #print(sort(table(Class), decreasing = TRUE))
  Classification <- list(Class, ClassLegend)
  names(Classification) <- c("Class", "ClassLegend")
  return(Classification)
}

#' scTHI_plotCluster
#'
#' Graphs the output of scTHI_runTsne, labeling cells by clusters.
#' @param scTHIresult scTHI object.
#' @param cexPoint Set the point size.
#' @param legendPos Character string to custom the legend position.
#' @examples
#' library(scTHI.data)
#' data(scExample)
#' result <-  scTHI_score(scExample,
#'                        cellCusterA = colnames(scExample)[1:30],
#'                        cellCusterB = colnames(scExample)[31:100],
#'                        cellCusterAName = "ClusterA",
#'                        cellCusterBName = "ClusterB", filterCutoff = 0,
#'                        pvalueCutoff = 1, nPermu = 100, ncore = 8)
#' result <- scTHI_runTsne(result)
#' scTHI_plotCluster(result)
#' @return None
#' @export
#'
#' scTHI_plotCluster

scTHI_plotCluster <- function(scTHIresult,
                              cexPoint = 0.8,
                              legendPos = c(
                                "topleft", "topright",
                                "bottomright", "bottomleft"
                              )) {
  
  tsneData <- scTHIresult$tsneData
  legendPos <- match.arg(legendPos)
  
  nCluster <- length(scTHIresult) - 3
  ggplot_colors <- function(n) {
    hues <- seq(15, 375, length = n + 1)
    hcl(h = hues, l = 65, c = 100)[1:n]
  }
  
  Colors <- ggplot_colors(n = nCluster)
  names(Colors) <- names(scTHIresult)[4:length(scTHIresult)]
  
  ClusterColors <- rep("gray80", nrow(tsneData))
  names(ClusterColors) <- rownames(tsneData)
  
  ClusterColors[scTHIresult[[names(Colors)[1]]]] <- Colors[1]
  ClusterColors[scTHIresult[[names(Colors)[2]]]] <- Colors[2]
  
  plot(
    tsneData[, 1:2],
    pch = 16,
    cex = cexPoint,
    col = ClusterColors[rownames(tsneData)],
    main = "",
    xlab = "tSNE 1",
    ylab = "tSNE 2"
  )
  legend(
    legendPos,
    inset = .02,
    legend = names(Colors),
    col = Colors,
    cex = 1,
    pch = 15,
    box.lty = 0,
    bg = "transparent"
  )
}

checkInteractions <- function(expMat){
  tmp_check <-
    interaction_table[, c(
      "partnerA1",
      "partnerA2",
      "partnerA3",
      "partnerB1",
      "partnerB2",
      "partnerB3"
    )]
  tmp_check <- unique(tmp_check[!is.na(tmp_check)])
  if (sum(tmp_check %in% rownames(expMat)) != length(tmp_check)) {
    tmp_genes <- setdiff(tmp_check, rownames(expMat))
    tmp <- which(
      interaction_table$partnerA1 %in% tmp_genes |
        interaction_table$partnerA2 %in% tmp_genes |
        interaction_table$partnerA3 %in% tmp_genes |
        interaction_table$partnerB1 %in% tmp_genes |
        interaction_table$partnerB2 %in% tmp_genes |
        interaction_table$partnerB3 %in% tmp_genes
    )
    interaction_table <- interaction_table[-tmp, ]
    message("Warning: Not all interaction genes are in expMat")
  }
  if (nrow(interaction_table) == 0) {
    stop("ERROR: No interaction genes to test")
  }
  return(interaction_table)
}

rankingValues <- function(x, topRank = 10){
  n <- round(nrow(x) * topRank / 100)
  ddata_ <- apply(x, 2, function(x) {
    rank(-x)
  })
  ddata_ <- apply(ddata_, 2, function(x) {
    x <= n
  })
  return(ddata_)
}

getScore <- function(expMat, interaction_table, cellCuster, 
                     genes1columns, genes2columns,autocrineEffect){
  ddata <- expMat[, cellCuster]
  ddata_ <- rankingValues(ddata)
  
  rnk <- rep(0, nrow(interaction_table))
  expValue <- rep(0, nrow(interaction_table))
  names(rnk) <- rownames(interaction_table)
  names(expValue) <- rownames(interaction_table)
  
  ans <-  apply(interaction_table, 1, function(x){
    ggenes1 <- x[genes1columns]
    ggenes1 <- as.vector(ggenes1[!is.na(ggenes1)])
    tmp_genes1 <- ddata_[ggenes1, , drop = FALSE]
    tmp1 <- colSums(tmp_genes1 == TRUE)
    
    if (autocrineEffect == FALSE) {
      ggenes2 <-
        x[genes2columns]
      ggenes2 <- as.vector(ggenes2[!is.na(ggenes2)])
      tmp_genes2 <- ddata_[ggenes2, , drop = FALSE]
      tmp2 <- colSums(tmp_genes2 == FALSE)
      tmp <- tmp1 + tmp2
      
      rnk<- round(sum(tmp == (nrow(tmp_genes1) +
                                nrow(tmp_genes2))) / ncol(tmp_genes1), digits = 2)
      
      expValue <-
        paste(round(rowMeans(ddata[ggenes1, cellCuster,
                                   drop = FALSE
                                   ]), digits = 2), collapse = " # ")
      
    }
    if (autocrineEffect == TRUE)  {
      tmp <- tmp1
      rnk <-
        round(sum(tmp == nrow(tmp_genes1)) / ncol(tmp_genes1),
              digits = 2
        )
      expValue <-
        paste(round(rowMeans(ddata[ggenes1, cellCuster,
                                   drop = FALSE
                                   ]), digits = 2), collapse = " # ")
    }
    list(rnk,expValue)
  })
  rresult <- data.frame(
    rnk = unlist(lapply(ans, function(x) x[[1]])),
    expValue = unlist(lapply(ans, function(x) x[[2]]))
  )
  
  return(rresult)
}


#' scTHI_score
#'
#' This function allows the user to compute a score for a set of
#' ligand-receptor pairs, from a single cell gene expression matrix,
#' and detect specific Tumor-Host interactions. You must specify at
#' least two clusters of cells (for example tumor cells and immune
#' cells).
#' @param expMat ScRNA-seq gene expression matrix where rows are genes
#'   presented with Hugo Symbols and columns are cells. Gene expression
#'   values should be counts or normalized counts.
#' @param cellCusterA Vector of columns of expMat that belong to the
#' first cluster.
#' @param cellCusterB Vector of columns of expMat that belong to the
#' second cluster.
#' @param cellCusterAName A character string labeling the clusterA.
#' @param cellCusterBName A character string labeling the clusterB.
#' @param topRank Filter threshold. Set to 10 (default) means that
#' each gene of the interaction pair will be considered as expressed
#' in a cell if it's in the top rank 10 percent.
#' @param autocrineEffect if TRUE remove the paracrine filter
#' @param fileNameBase Project name.
#' @param filterCutoff Score threshold (default is 0.50). For each
#'   interaction pair, if the score calculated (for the partnerA
#'   or partnerB)
#'   will be less than filterCutoff the interaction pair will be
#'   discarded.
#' @param PValue Logical, set to TRUE (default) compute statistical
#'   iterations. If p.value < 0.05, the value will be returned.
#' @param pvalueCutoff cutoff of the p-value
#' @param nPermu Number of iterations to perform (default is 1000).
#' @param ncore Number of processors to use.
#' @keywords interaction
#' @examples
#'
#' ####################### example of scTHI_score
#' library(scTHI.data)
#' data(scExample)
#' result <-  scTHI_score(scExample,
#'       cellCusterA = colnames(scExample)[1:30],
#'       cellCusterB = colnames(scExample)[31:100],
#'       cellCusterAName = "ClusterA",
#'       cellCusterBName = "ClusterB", filterCutoff = 0,
#'      pvalueCutoff = 1, nPermu = 100, ncore = 8)
#'
#' @return A list of results, with four items: result (data.frame),
#'   expMat (matrix), clusterA (character),  clusterA (character)
#' @export
#' scTHI_score

scTHI_score <-
  function(expMat,
           cellCusterA,
           cellCusterB,
           cellCusterAName,
           cellCusterBName,
           topRank = 10,
           autocrineEffect = TRUE,
           fileNameBase = "scTHI",
           filterCutoff = 0.5,
           PValue = TRUE,
           pvalueCutoff = 0.05,
           nPermu = 1000,
           ncore = 8) {
    
   
    ############ check all interaction genes in expMat ############
    interaction_table <- checkInteractions(expMat)
    
    #################### scTHI score ##############################
    message(paste(
      "Computing score for",
      nrow(interaction_table),
      "interaction pairs..."
    ))
    
    ##### ddataA
    resultA <- getScore(expMat, interaction_table = interaction_table,
                        cellCuster = cellCusterA,
                        genes1columns= c("partnerA1", "partnerA2", 
                                         "partnerA3"),
                        genes2columns=c("partnerB1", "partnerB2", 
                                        "partnerB3"),
                        autocrineEffect)
    colnames(resultA) <-
      paste0(c("rnkPartnerA", "expValueA"), "_", cellCusterAName)
    print(paste("Computed ranked values for partner A"))
    
    ###### ddataB
    resultB <- getScore(expMat, interaction_table = interaction_table,
                        cellCuster = cellCusterB,
                        genes1columns= c("partnerB1", "partnerB2", 
                                         "partnerB3"),
                        genes2columns=c("partnerA1", "partnerA2", 
                                        "partnerA3"),
                        autocrineEffect)
    colnames(resultB) <-
      paste0(c("rnkPartnerB", "expValueB"), "_", cellCusterBName)
    print(paste("Computed ranked values for partner B"))
    
    
    #################### merge results #################
    SCORE <- rowMeans(cbind(resultA$rnkPartnerA, resultB$rnkPartnerB))
    result <- data.frame(interaction_table, resultA, resultB, SCORE)
    
    ############### filter low-score int ###############
    message("Removing low score interactions...")
    interestColumn <- c(
      paste0("rnkPartnerA_", cellCusterAName),
      paste0("rnkPartnerB_", cellCusterBName)
    )
    tmp <- rowSums(result[, interestColumn] < filterCutoff)
    result[tmp != 0, "SCORE"] <- NA
    result <- result[!is.na(result$SCORE), ]
    result <- result[order(result$SCORE, decreasing = TRUE), ]
    result <- data.frame(
      interationPair = rownames(result),
      result,
      stringsAsFactors = FALSE
    )
    columnToremove <- c(
      "partnerA1",
      "partnerA2",
      "partnerA3",
      "partnerB1",
      "partnerB2",
      "partnerB3"
    )
    result <- result[, setdiff(colnames(result), columnToremove)]
    
    if (nrow(result) == 0) {
      stop("No interaction pair exceed the score filterCutoff")
    }
    
    scTHIresult <- list(result, expMat, cellCusterA, cellCusterB)
    names(scTHIresult) <-
      c("result", "expMat", cellCusterAName, cellCusterBName)
    
    ###################### Permutation #########
    if (PValue == TRUE) {
      message("Computing permutation....")
      interaction_table <- interaction_table[rownames(result), ]
      columnToadd <- c(
        "partnerA1",
        "partnerA2",
        "partnerA3",
        "partnerB1",
        "partnerB2",
        "partnerB3"
      )
      result <- data.frame(interaction_table[, columnToadd], result,
                           stringsAsFactors = FALSE
      )
      
      if (length(grep("simple", result$interactionType)) != 0) {
        sample_pairInteraction <-
          rownames(result)[result$interactionType == "simple"][1]
        
        param <- SnowParam(workers = ncore, type = "SOCK")
        if (autocrineEffect == FALSE) {
          paracrineFun <- function(dummy, rankingValues) {
            ddataA <- expMat[, cellCusterA]
            ddataA <-
              apply(ddataA, 2, function(x) {
                sample(x, replace = FALSE)
              })
            rownames(ddataA) <- rownames(expMat)
            ddataA_ <- rankingValues(ddataA)
            
            ggenes1 <-
              result[sample_pairInteraction, c(
                "partnerA1", "partnerA2",
                "partnerA3"
              )]
            ggenes1 <- as.vector(ggenes1[!is.na(ggenes1)])
            tmp_genes1 <- ddataA_[ggenes1, , drop = FALSE]
            tmp1 <- colSums(tmp_genes1 == TRUE)
            ggenes2 <-
              result[sample_pairInteraction, c(
                "partnerB1", "partnerB2",
                "partnerB3"
              )]
            ggenes2 <- as.vector(ggenes2[!is.na(ggenes2)])
            tmp_genes2 <- ddataA_[ggenes2, , drop = FALSE]
            tmp2 <- colSums(tmp_genes2 == FALSE)
            tmp <- tmp1 + tmp2
            rnk_permuA <- round(sum(tmp == (
              nrow(tmp_genes1) +
                nrow(tmp_genes2)
            )) / ncol(tmp_genes1), digits = 2)
            
            
            ddataB <- expMat[, cellCusterB]
            ddataB <-
              apply(ddataB, 2, function(x) {
                sample(x, replace = FALSE)
              })
            rownames(ddataB) <- rownames(expMat)
            ddataB_ <- rankingValues(ddataB)
            
            ggenes1 <-
              result[sample_pairInteraction, c(
                "partnerB1", "partnerB2",
                "partnerB3"
              )]
            ggenes1 <- as.vector(ggenes1[!is.na(ggenes1)])
            tmp_genes1 <- ddataB_[ggenes1, , drop = FALSE]
            tmp1 <- colSums(tmp_genes1 == TRUE)
            
            ggenes2 <-
              result[sample_pairInteraction, c(
                "partnerA1", "partnerA2",
                "partnerA3"
              )]
            ggenes2 <- as.vector(ggenes2[!is.na(ggenes2)])
            tmp_genes2 <- ddataB_[ggenes2, , drop = FALSE]
            tmp2 <- colSums(tmp_genes2 == FALSE)
            tmp <- tmp1 + tmp2
            
            rnk_permuB <- round(sum(tmp == (
              nrow(tmp_genes1) +
                nrow(tmp_genes2)
            )) / ncol(tmp_genes1), digits = 2)
            rnkInteraction <- mean(rnk_permuA, rnk_permuB)
            return(rnkInteraction)
          }
          ans <- bplapply(1:nPermu,
                          FUN = paracrineFun,
                          rankingValues = rankingValues,
                          BPPARAM = param
                          
          )
          ans <- unlist(ans)
          permutatedPvalue_simple <- ans
        }
        if (autocrineEffect == TRUE) {
          autocrineFun <- function(dummy, rankingValues) {
            ddataA <- expMat[, cellCusterA]
            ddataA <-
              apply(ddataA, 2, function(x) {
                sample(x, replace = FALSE)
              })
            rownames(ddataA) <- rownames(expMat)
            ddataA_ <- rankingValues(ddataA)
            
            ggenes1 <-
              result[sample_pairInteraction, c(
                "partnerA1", "partnerA2",
                "partnerA3"
              )]
            ggenes1 <- as.vector(ggenes1[!is.na(ggenes1)])
            tmp_genes1 <- ddataA_[ggenes1, , drop = FALSE]
            tmp1 <- colSums(tmp_genes1 == TRUE)
            tmp <- tmp1
            rnk_permuA <-
              round(sum(tmp == nrow(tmp_genes1)) / ncol(tmp_genes1),
                    digits = 2
              )
            
            
            ddataB <- expMat[, cellCusterB]
            ddataB <-
              apply(ddataB, 2, function(x) {
                sample(x, replace = FALSE)
              })
            rownames(ddataB) <- rownames(expMat)
            ddataB_ <- rankingValues(ddataB)
            
            ggenes1 <-
              result[sample_pairInteraction, c(
                "partnerB1", "partnerB2",
                "partnerB3"
              )]
            ggenes1 <- as.vector(ggenes1[!is.na(ggenes1)])
            tmp_genes1 <- ddataB_[ggenes1, , drop = FALSE]
            tmp1 <- colSums(tmp_genes1 == TRUE)
            tmp <- tmp1
            
            rnk_permuB <-
              round(sum(tmp == nrow(tmp_genes1)) / ncol(tmp_genes1),
                    digits = 2
              )
            rnkInteraction <- mean(rnk_permuA, rnk_permuB)
            return(rnkInteraction)
          }
          ans <- bplapply(1:nPermu,
                          FUN = autocrineFun,
                          rankingValues = rankingValues,
                          BPPARAM = param
                          
          )
          ans <- unlist(ans)
          permutatedPvalue_simple <- ans
        }
      }
      
      if (length(grep("complex", result$interactionType)) != 0) {
        sample_pairInteraction <-
          rownames(result)[result$interactionType == "complex"][1]
        
        
        param <- SnowParam(workers = ncore, type = "SOCK")
        if (autocrineEffect == FALSE) {
          paracrineFun2 <- function(dummy, rankingValues) {
            ddataA <- expMat[, cellCusterA]
            ddataA <-
              apply(ddataA, 2, function(x) {
                sample(x, replace = FALSE)
              })
            rownames(ddataA) <- rownames(expMat)
            ddataA_ <- rankingValues(ddataA)
            
            ggenes1 <-
              result[sample_pairInteraction, c(
                "partnerA1", "partnerA2",
                "partnerA3"
              )]
            ggenes1 <- as.vector(ggenes1[!is.na(ggenes1)])
            tmp_genes1 <- ddataA_[ggenes1, , drop = FALSE]
            tmp1 <- colSums(tmp_genes1 == TRUE)
            
            ggenes2 <-
              result[sample_pairInteraction, c(
                "partnerB1", "partnerB2",
                "partnerB3"
              )]
            ggenes2 <- as.vector(ggenes2[!is.na(ggenes2)])
            tmp_genes2 <- ddataA_[ggenes2, , drop = FALSE]
            tmp2 <- colSums(tmp_genes2 == FALSE)
            
            tmp <- tmp1 + tmp2
            rnk_permuA <- round(sum(tmp == (
              nrow(tmp_genes1) +
                nrow(tmp_genes2)
            )) / ncol(tmp_genes1),
            digits = 2
            )
            
            
            ddataB <- expMat[, cellCusterB]
            ddataB <-
              apply(ddataB, 2, function(x) {
                sample(x, replace = FALSE)
              })
            rownames(ddataB) <- rownames(expMat)
            ddataB_ <- rankingValues(ddataB)
            
            ggenes1 <-
              result[sample_pairInteraction, c(
                "partnerB1", "partnerB2",
                "partnerB3"
              )]
            ggenes1 <- as.vector(ggenes1[!is.na(ggenes1)])
            tmp_genes1 <- ddataB_[ggenes1, , drop = FALSE]
            tmp1 <- colSums(tmp_genes1 == TRUE)
            
            ggenes2 <-
              result[sample_pairInteraction, c(
                "partnerA1", "partnerA2",
                "partnerA3"
              )]
            ggenes2 <- as.vector(ggenes2[!is.na(ggenes2)])
            tmp_genes2 <- ddataB_[ggenes2, , drop = FALSE]
            tmp2 <- colSums(tmp_genes2 == FALSE)
            tmp <- tmp1 + tmp2
            
            rnk_permuB <- round(sum(tmp == (
              nrow(tmp_genes1) +
                nrow(tmp_genes2)
            )) / ncol(tmp_genes1), digits = 2)
            rnkInteraction <- mean(rnk_permuA, rnk_permuB)
            return(rnkInteraction)
          }
          ans <- bplapply(1:nPermu,
                          FUN = paracrineFun2,
                          rankingValues = rankingValues,
                          BPPARAM = param
                          
          )
          ans <- unlist(ans)
          permutatedPvalue_complex <- ans
        }
        if (autocrineEffect == TRUE) {
          autocrineFun2 <- function(dummy, rankingValues) {
            ddataA <- expMat[, cellCusterA]
            ddataA <-
              apply(ddataA, 2, function(x) {
                sample(x, replace = FALSE)
              })
            rownames(ddataA) <- rownames(expMat)
            ddataA_ <- rankingValues(ddataA)
            
            ggenes1 <-
              result[sample_pairInteraction, c(
                "partnerA1", "partnerA2",
                "partnerA3"
              )]
            ggenes1 <- as.vector(ggenes1[!is.na(ggenes1)])
            tmp_genes1 <- ddataA_[ggenes1, , drop = FALSE]
            tmp1 <- colSums(tmp_genes1 == TRUE)
            tmp <- tmp1
            rnk_permuA <-
              round(sum(tmp == nrow(tmp_genes1)) / ncol(tmp_genes1),
                    digits = 2
              )
            
            
            ddataB <- expMat[, cellCusterB]
            ddataB <-
              apply(ddataB, 2, function(x) {
                sample(x, replace = FALSE)
              })
            rownames(ddataB) <- rownames(expMat)
            ddataB_ <- rankingValues(ddataB)
            
            ggenes1 <-
              result[sample_pairInteraction, c(
                "partnerB1", "partnerB2",
                "partnerB3"
              )]
            ggenes1 <- as.vector(ggenes1[!is.na(ggenes1)])
            tmp_genes1 <- ddataB_[ggenes1, , drop = FALSE]
            tmp1 <- colSums(tmp_genes1 == TRUE)
            tmp <- tmp1
            
            rnk_permuB <-
              round(sum(tmp == nrow(tmp_genes1)) / ncol(tmp_genes1),
                    digits = 2
              )
            rnkInteraction <- mean(rnk_permuA, rnk_permuB)
            return(rnkInteraction)
          }
          ans <- bplapply(1:nPermu,
                          FUN = autocrineFun2,
                          rankingValues = rankingValues,
                          BPPARAM = param
                          
          )
          ans <- unlist(ans)
          permutatedPvalue_complex <- ans
        }
      }
      
      
      
      r <- apply(result, 1, function (x) {
        if (x["interactionType"] == "simple") {
          ans <-    sum(permutatedPvalue_simple > x["SCORE"]) / nPermu
        }
        if (x["interactionType"] == "complex") {
          ans <- sum(permutatedPvalue_complex > x["SCORE"]) / nPermu
        }
        ans
      })
      SCOREpValue <- unlist(r)
      FDR <- round(p.adjust(SCOREpValue, method = "fdr"), digits = 3)
      result <- data.frame(
        result,
        pValue = SCOREpValue,
        FDR = FDR,
        stringsAsFactors = FALSE
      )
      
      result <- result[result$pValue <= pvalueCutoff, ]
      columnToremove <- c(
        "partnerA1",
        "partnerA2",
        "partnerA3",
        "partnerB1",
        "partnerB2",
        "partnerB3"
      )
      result <- result[, setdiff(colnames(result), columnToremove)]
      scTHIresult$result <- result
      # save(scTHIresult, file = paste0(fileNameBase,
      # "_", cellCusterAName,
      #  "&", cellCusterBName, ".RData"))
    }
    message("Interaction pairs detected:")
    print(rownames(scTHIresult$result))
    return(scTHIresult)
  }

---
title: "Using scTHI"
author: 
- name:    "Michele Ceccarelli" 
- name:    "Francesca Pia Caruso"
output:    BiocStyle::html_document 
vignette: >
    %\VignetteIndexEntry{Using scTHI}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}  
    %\VignetteDepends{BiocStyle} 
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```



# Introduction
The detection of tumor-host interaction is a major challange in cancer 
understanding. More recent advances in single cell next generation 
sequencing have provided new insights in unveiling tumor microenvironment 
(TME) cells comunications. The scTHI package provides some useful functions 
to identify and visualize the cell types making up the TME, and a rank-based 
approach to discover interesting ligand-receptor interactions establishing 
among malignant and non tumor cells

# Installation

```{r echo = FALSE, message=FALSE,warning=FALSE, eval=FALSE, echo=TRUE}
if(!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("scTHI")
```

# Examples 
## Analysis of H3K27M glioma from Fibin et al. Science 2018

### Input Data
The scTHI package requires a matrix of count data (or normalized counts) 
from single cell RNA-seq experiments, where rows are genes presented with 
Hugo Symbols and columns are cells. For a practical demonstration, we 
will use the scRNA-seq data available on the [Broad Institute Single-Cell 
Portal](https://portals.broadinstitute.org/single_cell/study/single-cell-
analysis-in-pediatric-midline-gliomas-withhistone-h3k27m-mutation) [1]. 
The dataset provides 3,321 scRNA-seq profiles from six primary Glioma 
(H3K27M-glioma), and includes both malignant cells and several tumor 
microenvironment cell types, as immune cells and oligodendrocytes. 
Our goal is to identify the significant ligand-receptor interactions 
that are established between cancer and immune cells present in the tumor
microenvironment. For this reason, after downloading H3K27M-glioma data 
we preprocessed them, selecting only one sample of interest (i.e. BCH836), 
removing not-expresse genes, transforming data in log TPM , and applying 
quantile normalization. Processed are included in the package and can be 
loaded as follows:

```{r,warning=FALSE}
library(scTHI)
library(scTHI.data)
set.seed(1234)
data("H3K27")
data("H3K27.meta")
```


H3K27.meta includes the annotation of each cell, so as to identify 
the tumor cells and the immune cells in BCH836 patient:

```{r}
table(H3K27.meta$Type)
```

We will use the cells annotated as Immune and a subset of Malignant.

```{r}
Malignant <-
rownames(H3K27.meta)[H3K27.meta$Type == "Malignant"][1:100]
Immune <- rownames(H3K27.meta)[H3K27.meta$Type == "Immune cell"]
```

### Run scTHI.score
The function `scTHI.score` can be used  to detect significant 
ligand-receptor interactions occurring between cancer and immune cells. 
The function uses a dataset of 1044 ligand-receptor pairs to identify 
those which are significantly enriched between pairs clusters of your 
dataset. A score is computed for each interaction pair. This score does 
not simply reflect the average expression of the two partners in the 
interaction pair, it is rather based on the percentage of cells that 
express the interaction pair at the top of the ordered gene list. It 
considers a gene expressed in a cell, only if it is in the top 10% of 
the ranked gene list, as set in `topRank` argument.  
Interactions are not symmetric, so partnerA expression is considered 
on the first cluster (in the example the Malignant cells), and partnerB 
expression is considered on the second cluster (i.e. Immune cells). 
A p-value for each pair is computed, based on a null model from random 
permutations of data, and  and only significant 
interaction pairs are reported.

```{r, warning=FALSE}
H3K27.result <-
scTHI_score(
expMat = H3K27,
cellCusterA = Malignant,
cellCusterB = Immune,
cellCusterAName = "Malignant",
cellCusterBName = "Immune",
topRank = 10,
PValue = TRUE,
pvalueCutoff = 0.05,
nPermu = 10,
ncore = 1
)
```
For which partner A and partner B are expressed in at least 50% 
of the cells of the respective clusters, as set in `filterCutoff` 
argument. P-value computation is optional and is estimated by 
permuting the mates of interaction pairs. Result also includes 
two columns with the average expression values of each partner
in the respective clusters.

```{r}
head(H3K27.result$result)
```

The function `scTHI.plotResult` with argument `plotType` as 
"score" displays the score of each significant  pair through 
barplots. The number of asterisks indicates significance. 
Otherwise, setting the argument `plotType` as "pair", for each 
pair are shown bar plots displaying the percentage of cells in 
each cluster expressing partner A and partner B, respectively.

```{r}
scTHI_plotResult(scTHIresult = H3K27.result,
                cexNames = 0.7,
                plotType = "score")
```


```{r}
scTHI_plotResult(scTHIresult = H3K27.result,
                cexNames = 0.7,
                plotType = "pair")
```

Another useful way to explore results is the visualization of the 
entire data set through the t-SNE plot. scTHI allows the user to 
compute the non-linear dimensionality reduction of data using the 
scTHI.runTsne function based on the Rtsne R package [2].
```{r}
H3K27.result <- scTHI_runTsne(scTHIresult = H3K27.result)
```

Users can decide to display on the tsne the two input clusters, 
i.e. Maglignant and Immune cells, or to observe the intensity of 
the expression of an intresting interaction pair in the whole 
dataset. The scTHI.plotCluster function differently labels cells 
on t-SNE if they belong to ClusterA or ClusterB.

```{r}
scTHI_plotCluster(scTHIresult = H3K27.result,
                cexPoint = 0.8,
                legendPos = "bottomleft")
```


On the other hand the scTHI.plotPairs function colors the cells on 
the t-SNE based on the expression levels of a user-defined 
interaction pair. For example, let’s try plotting the expression 
of the interaction pair consisting of THY1, which should be expressed 
by the cancer cells and the ITGAX-ITGB2 complex, instead expressed 
by the immune cells.

```{r}
scTHI_plotPairs(
scTHIresult = H3K27.result,
cexPoint = 0.8,
interactionToplot = "THY1_ITGAX:ITGB2"
)
```

The THY1 gene appears uniformly expressed in tumor cells, and 
in according to results table, it was expressed in 861% of 
malignant cells at the top 10% of the ranked gene list of 
each cell. On the other hand the ITGAX-ITGB2 complex is highly 
specific of the Immune cluster, where it is expressed by 92% of
the cells at the top 10% of the ranked gene list of each cell.

### Tumor microenvironment classification
Another important problem in study tumor-host interaction is 
the classification of all cell types that make up the tumor 
microenvironment. Generally, tools for single cell data analysis 
are based on the use of marker genes to classify different cell 
types identified by clustering approaches. This strategy is not 
always the most accurate, our approach uses a different method 
for the classification of TME based on gene signature enrichment 
analysis. The `TME.classification` implements the 
Mann-Whitney-Wilcoxon Gene Set Test (MWW-GST) algorithm [3] 
and tests for each cell the enrichment of a collection of 
signatures of different cell types, with greater interest for 
the Immune System and the Central Nervous System cells. Thus, 
each cell is associated with a specific phenotype based on the 
more significant enriched gene set. 
Let's try to classify the TME cells of the BCH836 patient, 
excluding malignant cells from the dataset.

```{r}
Tme.cells <-
rownames(H3K27.meta)[which(H3K27.meta$Type != "Malignant")]
H3K27.tme <- H3K27[, Tme.cells]
```
Note that `TME.classification` function can take a 
long time if the number of cells to test is large.



```{r, message=FALSE}
Class <- TME_classification(expMat = H3K27.tme, minLenGeneSet = 10)
```
The H3K27M dataset annotation table suggests that non-tumor cells 
from BCH836 patient are mainly composed by Oligodendrocyte and Immune 
cells, respectively. The TME.classification function identifies a 
similar number of Oligodendrocytes and Immune cells, distinguishing 
the latter mainly in Microglia, Monocytes and Macrophages and fewer 
other Immune cells.

```{r}
table(Class$Class[Tme.cells], H3K27.meta[Tme.cells, "Type"])
```
Finally, using the previously calculated t-SNE or a set of 
user-defined coordinates, you can view the new classification 
of the TME.
```{r}
TME_plot(tsneData = H3K27.result$tsneData, Class, cexPoint = 0.8)
```


# Session Info
```{r}
sessionInfo()
```

# References
1. Filbin, M. G., Tirosh, I., Hovestadt, V., Shaw, M. L., Escalante, 
L. E., Mathewson, N. D., ... & Haberler, C. (2018). Developmental 
and oncogenic programs in H3K27M gliomas dissected by single-cell 
RNA-seq. Science, 360(6386), 331-335.


2. Maaten, L. V. D., & Hinton, G. (2008). Visualizing data 
using t-SNE. Journal of machine learning research, 9(Nov), 2579-2605.

3. Frattini, V., Pagnotta, S. M., Fan, J. J., Russo, M. V., Lee, 
S. B., Garofano, L., ... & Frederick, V. (2018). A metabolic 
function of FGFR3-TACC3 gene fusions in cancer. Nature, 553(7687), 222.

---
title: "External data"
output: html_document
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

# Interaction table

The object `interaction_table` contains a list of 2,548 pairs of ligand and receptors. Data has been collected curating publicly available resources from:

 * Ramilowski JA, Goldberg T, Harshbarger J, Kloppmann E, Lizio M, Satagopam VP, et al. A draft
network of ligand-receptor-mediated multicellular signalling in human. Nat Commun. 2015;6:7866.
doi:10.1038/ncomms8866.

 * Vento-Tormo R, Efremova M, Botting RA, Turco MY, Vento-Tormo M, Meyer KB, et al. Single-cell
reconstruction of the early maternal-fetal interface in humans. Nature. 2018;563:347–53.
doi:10.1038/s41586-018-0698-6.

The curated list is composed of known and novel literature-supported interactions and includes both heteromeric and monomeric ligands/receptors mainly related to chemokine, cytokine, growth factors, integrin, TGF and TNF family members, semaphorins, ephrins, Wnt and Notch signalings.. 



# Signatures

The object `signatures` contains a list of 295 cell-type specific signatures, including the Immune and Central Nervous Systems. Data has been collected integrating a cureted collection of marker genes and a set of signatures available from public databases and published studies, as:

 * a compendium of 64 human cell types signatures including lymphoid, myeloid, stromal, tissue-specific and stem cells, collected from FANTOM5, ENCODE, Blueprint and Gene Expression Omnibus (GEO) data portals; 
 
 * a set of markers for 30 immune cell types, including myeloid and lymphoid subpopulations identified from Peripheral Blood Mononuclear Cells (PBMCs) [1]; 
 
 * a set of Central Nervous System cell signatures including astrocytes, neuron, oligodendrocytes, microglia, and endothelial cells [2];
 
 * a set of 53 signatures corresponding to 26 different cell types [3-6]
 
 * two gene expression programs related to microglia and bone marrow-derived macrophages in gliomas [7]. 


The object `signaturesColors` is a data.frame including a list of colors for `signatures`.


##References

[1] Butler A, Hoffman P, Smibert P, Papalexi E, Satija R. Integrating single-cell transcriptomic data
across different conditions, technologies, and species. Nat Biotechnol. 2018;36:411–20.
doi:10.1038/nbt.4096.


[2] Zhang Y, Chen K, Sloan SA, Bennett ML, Scholze AR, O’Keeffe S, et al. An RNA-sequencing
transcriptome and splicing database of glia, neurons, and vascular cells of the cerebral cortex. J
Neurosci. 2014;34:11929–47. doi:10.1523/JNEUROSCI.1860-14.2014.


[3] Bindea G, Mlecnik B, Tosolini M, Kirilovsky A, Waldner M, Obenauf AC, et al. Spatiotemporal
dynamics of intratumoral immune cells reveal the immune landscape in human cancer. Immunity.
2013;39:782–95. doi:10.1016/j.immuni.2013.10.003.


[4] Charoentong P, Finotello F, Angelova M, Mayer C, Efremova M, Rieder D, et al. Pan-cancer
Immunogenomic Analyses Reveal Genotype-Immunophenotype Relationships and Predictors of
Response to Checkpoint Blockade. Cell Rep. 2017;18:248–62. doi:10.1016/j.celrep.2016.12.019.


[5] Rooney MS, Shukla SA, Wu CJ, Getz G, Hacohen N. Molecular and genetic properties of tumors
associated with local immune cytolytic activity. Cell. 2015;160:48–61. doi:10.1016/j.cell.2014.12.033.


[6] Tirosh I, Izar B, Prakadan SM, Wadsworth MH, Treacy D, Trombetta JJ, et al. Dissecting the
multicellular ecosystem of metastatic melanoma by single-cell RNA-seq. Science. 2016;352:189–96.
doi:10.1126/science.aad0501.


[7] Venteicher AS, Tirosh I, Hebert C, Yizhak K, Neftel C, Filbin MG, et al. Decoupling genetics,
lineages, and microenvironment in IDH-mutant gliomas by single-cell RNA-seq. Science. 2017;355.
doi:10.1126/science.aai8478.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scTHI_plotPairs.R
\name{scTHI_plotPairs}
\alias{scTHI_plotPairs}
\title{scTHI_plotPairs}
\usage{
scTHI_plotPairs(scTHIresult, cexPoint = 0.8, interactionToplot)
}
\arguments{
\item{scTHIresult}{scTHI object.}

\item{cexPoint}{Set the point size.}

\item{interactionToplot}{Interaction pair to plot.}
}
\value{
None
}
\description{
Generates a plot on the t-SNE coordinates to
show the expression levels
of an interaction pair of interest. Each cell is
colored according to the
corresponding
gene expression value.
}
\examples{
library(scTHI.data)
data(scExample)
result <-  scTHI_score(scExample,
                 cellCusterA = colnames(scExample)[1:30],
                 cellCusterB = colnames(scExample)[31:100],
                 cellCusterAName = "ClusterA",
                 cellCusterBName = "ClusterB", filterCutoff = 0,
                 pvalueCutoff = 1, nPermu = 100, ncore = 8)
result <- scTHI_runTsne(result)
scTHI_plotPairs(result,interactionToplot = "CXCL12_CD4")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TME_plot.R
\name{TME_plot}
\alias{TME_plot}
\title{TME_plot}
\usage{
TME_plot(tsneData, Class, cexPoint = 0.8)
}
\arguments{
\item{tsneData}{X and y coordinates of points in the plot.}

\item{Class}{Object returned by TME_classification function.}

\item{cexPoint}{Set the point size.}
}
\value{
None
}
\description{
Generates a plot on the t-SNE coordinates, labeling cells by TME
classification.
}
\examples{
library(scTHI.data)
data(scExample)
result <-  scTHI_score(scExample,
           cellCusterA = colnames(scExample)[1:30],
           cellCusterB = colnames(scExample)[31:100],
           cellCusterAName = "ClusterA",
           cellCusterBName = "ClusterB", filterCutoff = 0,
           pvalueCutoff = 1, nPermu = 100, ncore = 8)
result <- scTHI_runTsne(result)
Class <- TME_classification(scExample)
TME_plot(tsneData = result$tsneData, Class)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scTHI_plotResult.R
\name{scTHI_plotResult}
\alias{scTHI_plotResult}
\title{scTHI_plotResult}
\usage{
scTHI_plotResult(scTHIresult, cexNames = 0.8, plotType = c("score",
  "pair"), nRes = NULL)
}
\arguments{
\item{scTHIresult}{scTHI object.}

\item{cexNames}{Size of names in barplot.}

\item{plotType}{Type of plot to be generated. Default is
"score", can be  also "pair". The "score" option will generate
a barplot for each resulted interaction pair, representing
the calculated interaction score
  and the related p-Value.The "pair" option will generate
  two barplot for each resulted interaction pair, representing
  the percentage of cells of each cluster expressing partnerA
  and partnerB gene, respectively.}

\item{nRes}{Number of pairs to plot (all if NULL).}
}
\value{
None
}
\description{
Creates barplots of scTHI_score results.
}
\examples{
library(scTHI.data)
data(scExample)
result <-  scTHI_score(scExample,
                       cellCusterA = colnames(scExample)[1:30],
                       cellCusterB = colnames(scExample)[31:100],
                       cellCusterAName = "ClusterA",
                       cellCusterBName = "ClusterB", filterCutoff = 0,
                       pvalueCutoff = 1, nPermu = 100, ncore = 8)

scTHI_plotResult(result, plotType = "score")
scTHI_plotResult(result, plotType = "pair")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scTHI_runTsne.R
\name{scTHI_runTsne}
\alias{scTHI_runTsne}
\title{scTHI_runTsne}
\usage{
scTHI_runTsne(scTHIresult)
}
\arguments{
\item{scTHIresult}{scTHI object.}
}
\value{
The same object as scTHI_score with a fifth item tsneData
  (data.frame)
}
\description{
Runs t-SNE dimensionality reduction on selected features based on Rtsne
package.
}
\examples{
library(scTHI.data)
data(scExample)
result <-  scTHI_score(scExample,
                       cellCusterA = colnames(scExample)[1:30],
                       cellCusterB = colnames(scExample)[31:100],
                       cellCusterAName = "ClusterA",
                       cellCusterBName = "ClusterB", filterCutoff = 0,
                       pvalueCutoff = 1, nPermu = 100, ncore = 8)
result <- scTHI_runTsne(result)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scTHI_plotCluster.R
\name{scTHI_plotCluster}
\alias{scTHI_plotCluster}
\title{scTHI_plotCluster}
\usage{
scTHI_plotCluster(scTHIresult, cexPoint = 0.8, legendPos = c("topleft",
  "topright", "bottomright", "bottomleft"))
}
\arguments{
\item{scTHIresult}{scTHI object.}

\item{cexPoint}{Set the point size.}

\item{legendPos}{Character string to custom the legend position.}
}
\value{
None
}
\description{
Graphs the output of scTHI_runTsne, labeling cells by clusters.
}
\examples{
library(scTHI.data)
data(scExample)
result <-  scTHI_score(scExample,
                       cellCusterA = colnames(scExample)[1:30],
                       cellCusterB = colnames(scExample)[31:100],
                       cellCusterAName = "ClusterA",
                       cellCusterBName = "ClusterB", filterCutoff = 0,
                       pvalueCutoff = 1, nPermu = 100, ncore = 8)
result <- scTHI_runTsne(result)
scTHI_plotCluster(result)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scTHI_score.R
\name{scTHI_score}
\alias{scTHI_score}
\title{scTHI_score}
\usage{
scTHI_score(expMat, cellCusterA, cellCusterB, cellCusterAName,
  cellCusterBName, topRank = 10, autocrineEffect = TRUE,
  fileNameBase = "scTHI", filterCutoff = 0.5, PValue = TRUE,
  pvalueCutoff = 0.05, nPermu = 1000, ncore = 8)
}
\arguments{
\item{expMat}{ScRNA-seq gene expression matrix where rows are genes
presented with Hugo Symbols and columns are cells. Gene expression
values should be counts or normalized counts.}

\item{cellCusterA}{Vector of columns of expMat that belong to the
first cluster.}

\item{cellCusterB}{Vector of columns of expMat that belong to the
second cluster.}

\item{cellCusterAName}{A character string labeling the clusterA.}

\item{cellCusterBName}{A character string labeling the clusterB.}

\item{topRank}{Filter threshold. Set to 10 (default) means that
each gene of the interaction pair will be considered as expressed
in a cell if it's in the top rank 10 percent.}

\item{autocrineEffect}{if TRUE remove the paracrine filter}

\item{fileNameBase}{Project name.}

\item{filterCutoff}{Score threshold (default is 0.50). For each
interaction pair, if the score calculated (for the partnerA
or partnerB)
will be less than filterCutoff the interaction pair will be
discarded.}

\item{PValue}{Logical, set to TRUE (default) compute statistical
iterations. If p.value < 0.05, the value will be returned.}

\item{pvalueCutoff}{cutoff of the p-value}

\item{nPermu}{Number of iterations to perform (default is 1000).}

\item{ncore}{Number of processors to use.}
}
\value{
A list of results, with four items: result (data.frame),
  expMat (matrix), clusterA (character),  clusterA (character)
}
\description{
This function allows the user to compute a score for a set of
ligand-receptor pairs, from a single cell gene expression matrix,
and detect specific Tumor-Host interactions. You must specify at
least two clusters of cells (for example tumor cells and immune
cells).
}
\examples{

####################### example of scTHI_score
library(scTHI.data)
data(scExample)
result <-  scTHI_score(scExample,
      cellCusterA = colnames(scExample)[1:30],
      cellCusterB = colnames(scExample)[31:100],
      cellCusterAName = "ClusterA",
      cellCusterBName = "ClusterB", filterCutoff = 0,
     pvalueCutoff = 1, nPermu = 100, ncore = 8)

}
\keyword{interaction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TME_classification.R
\name{TME_classification}
\alias{TME_classification}
\title{TME_classification}
\usage{
TME_classification(expMat, minLenGeneSet = 10,
  alternative = "two.sided", pvalFilter = FALSE, fdrFilter = TRUE,
  pvalCutoff = 0.01, nesCutoff = 0.58, nNES = 1)
}
\arguments{
\item{expMat}{Gene expression matrix where rows are genes
presented with
Hugo Symbols and columns are cells. Gene expression values
should be normalized counts.}

\item{minLenGeneSet}{Minimum gene set length}

\item{alternative}{a character string specifying the alternative
hypothesis of wilcoxon test, must be one of "two.sided" (default),
"greater" or "less".}

\item{pvalFilter}{Logical, if TRUE results will be filtered
for p-Value.
  Defoult is FALSE.}

\item{fdrFilter}{Logical, if TRUE results will be filtered for FDR.}

\item{pvalCutoff}{Numeric p-Value (or FDR) threshold. Gene set with
p-Value (or FDR) greater than pvalCutoff will be discarded
(default is 0.01).}

\item{nesCutoff}{Numeric threshold. Gene set with NES greater than
nesCutoff will be discarded (default is 0.58)}

\item{nNES}{Default is 0.58, so each cell is classified with
a specific
 phenotype based on the first significant enriched gene set.}
}
\value{
A list with two items: Class (character) and ClassLegend
  (character)
}
\description{
The function allows the user to classify non-tumor cells in tumor
microenvironment. It implements the Mann-Whitney-Wilcoxon Gene
Set Test
(MWW-GST) algorithm and tests for each cell the enrichment of
a collection
of signatures of different cell types.
}
\examples{
library(scTHI.data)
data(scExample)
Class <- TME_classification(scExample)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scTHI.R
\docType{package}
\name{scTHI}
\alias{scTHI}
\alias{scTHI-package}
\title{single cell Tumor Hist Interaction (scTHI)}
\description{
Indentification of  significantly activated ligand-receptor 
interactions across clusters of cells from single-cell RNA sequencing data.
Single-cell RNA sequencing is the reference technique to characterize the heterogeneity of tumor microenvironment. 
The composition of the various cell types making up the microenvironment can significantly affect the way 
in which the immune system activates cancer rejection mechanisms. Understanding the cross-talk signals
 between immune cells and cancer cells is a fundamental for the identification immuno-oncology therapeutic targets. 
scTHI is  a novel method, single cell Tumor-Host Interaction tool (scTHI), 
to identify significantly activated ligand-receptor interactions across clusters of cells from 
single-cell RNA sequencing data. 
cTHI is based on the hypothesis that when patterns of interaction are active, 
they are also simultaneously and highly expressed in
homogeneous cell populations. We also model the autocrine and paracrine 
signalling effects of L-R partners
}
\details{
Please have a look at the vignette for a in-depth introduction to the package.
}
